(*****************************************)
(*    AXXABSolve - function              *)
(*****************************************)
BeginPackage["Polynomial`AXXABSolve`","Polynomial`Object`","Polynomial`Common`"]
(* U S A G E *)

AXXABSolve::usage = "Solves the bilateral symmetric matrix equation A'X + X'A = B 
                     \n where A is a square matrix
                     \n and B is a symmetric (Hermitian) matrix
                     \n B = B'";
(* O P T I O N S *)
(* M E S S A G E *) 
   General::varerr = " Different variables! ";
   General::nosol = " No solution! ";
   General::notsym = " Second argument is not symmetric polynom! ";
   General::argerr = " Arguments are not polynomial objects! ";
   General::arg1err = " First argument is not polynomial object! ";
   General::arg2err = " Second argument is not polynomial object! ";
   
Begin["`Private`"]

AXXABSolve[A:Poly[pcA_,varA_,degA_],B:Poly[pcB_,varB_,degB_]] :=
Module[{
         lb, k, i, j,
         mls, sol, var
       },

   If[varA =!= varB, Message[General::varerr, varA]];
   var = varA;

   If[
(* !! CHOP *)Not[And @@ (If[(Chop[#] === 0 || # === 0.), True, False] & /@ Last /@ Partition[pcB, 2])], 
      Message[General::notsym]; Return[Null]
   ];
  
   lb = PadLeft[pcB, degB+1, 0];

   If[degA <= degB/2,
   (* then *)
	   lb = Partition[lb, 1, 2];
	   k = PadRight[ PadLeft[pcA, degA+1, 0], {degB + 1}];
	   i = degB/2 + 1; (* pocet rovnic soustavy (= stupen b / 2 + 1) *) 
	   j = degB - degA + 1, (* pocet neznamych soustavy (= stupen x + 1) *)
   (* else *)
	   lb = PadRight[ Partition[lb, 1, 2], {degA}, {{0}} ];
	   k = PadRight[ PadLeft[pcA, degA+1, 0], {2*degA - 1}];
	   i = degA;
	   j = degA;
   ];    
  
  (* SylvestrMatrix *) 
            (* 0 ? 0.! *)
	mls = Table[0, {i}, {j}];
	
	Do[
		If[ jj <= 2*ii - 1,
	   	If[ EvenQ[jj],
				mls[[ii, jj]] = -2 k[[2*ii - 1 - jj + 1]],
				mls[[ii, jj]] = 2 k[[2*ii - 1 - jj + 1]]
			]
		]
	, {ii, i}, {jj, j}
	];	

	
   If[Global`$Verbose,
      PrintVerbose["Verbose output for equation a*.x+x*.a = b with scalar polynoms."];
      PrintVerbose["Sylvestr matrix for a:"];
      PrintVerbose[mls//MatrixForm];
      PrintVerbose["Sylvestr matrix for b:"];
      PrintVerbose[lb//MatrixForm];
   ];


(* pomalejsi pro realna cisla *)	
(*	
   mls = 
	Array[
		If[ #2 > 2*#1 - 1,
		0,
			If[ EvenQ[#2],
			-2 k[[2*#1 - 1 - #2 + 1]],
			2 k[[2*#1 - 1 - #2 + 1]]
			]
		] &,
	{i, j}];	
*)	

   sol = LinearSolve[mls, lb];

   If[Head[sol] === LinearSolve, Message[General::nosol]; Return[Null]];
   
   Poly[Flatten[sol], var, j-1]
];
(* error arguments *)

AXXABSolve[Null,B_] := (Message[General::arg1err];Return[Null]);
AXXABSolve[A_,Null] := (Message[General::arg2err];Return[Null]);
AXXABSolve[Null,Null] := (Message[General::argerr];Return[Null]);

End[]
 
EndPackage[]   