(*****************************************)
(*    USEFUL - function                  *)
(*****************************************)
BeginPackage["Polynomial`Common`","Polynomial`Object`"]
(* U S A G E *)
PrintVerbose::usage = " Print verbose. ";
DropLastZeros::usage = " Drop last zeros. ";
DropLastZeros0::usage = " Drop last zeros. ";
PolyQ::usage = " Poly[pol:polyobj] .. tests if input is polynomial object. "

Deg::usage = " Deg[A_,var_, opts___?OptionQ] - Various degrees matrices of a polynomial matrix.
        \n\t D = Deg[A,var] default, the same as DEG[A,var,TypeDegree->\"mat\"].
        \n\t D = Deg[A,var,TypeDegree->\"mat\"] returns the degree of polynomial matrix A.
        \n\t D = Deg[A,var,TypeDegree->\"row\"] returns the vector of row degrees of A.
        \n\t D = Deg[A,var,TypeDegree->\"col\"] returns the vector of column degrees of A. ";

SquareQ::usage = " Tests if polynomial object is square matrix. ";

SylvestrMatrix::usage = " SylvestrMatrix[pol] returns Sylvestr matrix of pol. ";

Star::usage = " Star. ";

Pol::usage = " Return polynoms. ";

IsPoly::usage = " Make symbol as poly object. ";
ListPoly::usage = " List of polynomials objects. ";

NonZeros::usage = " Select non zeros elements from the list. ";

SymbolQ::usage = " Test if input is symbol. ";

Size::usage = " Return size of polynomial object. ";

(* O P T I O N S *)
    Options[Deg] = {TypeDegree -> "mat"}
    
(* M E S S A G E *) 
    Deg::invopt = " Invalid options TypeDegree. ";
    Square::pol = " Argument is polynom! ";
    PolyQ::argerr = " One argument expect. ";
    General::difvars = "Different variable in polynomial matrix and scalar polynom: `1`.";
 
Begin["`Private`"]   
(* Developer context *)
<<Developer`

(******************************************************************************)
(*         U S E F U L    F U N C T I O N S                                   *)
(******************************************************************************)
Pol[A:Poly[pc_,var_,deg_]] := Plus @@ Table[pc[[i + 1]]*var^i, {i, 0, deg}];
Pol[A:PolyMat[pmc_,var_,deg_]] := Plus @@ Table[pmc[[i + 1]]*var^i, {i, 0, deg}];

Variable[A:(_Poly|_PolyMat)] := A[[2]];


(******************************************************************************)
(*          F U N C T I O N S                                                 *)
(******************************************************************************)
(* PolyQ[pol] ... Return True or False *)
(* tests if input object is polynomial object *)
PolyQ[pol:(_Poly|_PolyMat)] := True;
PolyQ[_] := False;
PolyQ[__] := ( Message[PolyQ::argerr]; Null );

(* List of polynomial objects *)
PolyData = {};
ListPoly[] := PolyData;

(* Set symbol as polynomial *)
IsPoly[syms__Symbol] := ((PolyQ[#] ^= True)& /@ {syms}; PolyData = Union[PolyData,{syms}]; True);
(* IsPoly[sym_Symbol]:=PolyQ[sym]^=True; *)


(********************************************************)
(*  various degrees matrices of a polynomial matrix.    *)
(********************************************************)
Deg[pol:PolyMat[pmc_,var_,deg_], opts___?OptionQ] :=
    Module[{n, dc, countzerovectors, countzero, typedegree = TypeDegree /. {opts} /. Options[Deg] },
        countzerovectors[dat_(*List of Lists*)] :=
           Module[{i = deg+1},
              While[And@@ZeroQ[dat[[i]]], i = i - 1]; 
              i - 1
           ];
        countzero[dat_ (*List*)] :=
           Module[{i = deg+1},
              While[ZeroQ[dat[[i]]], i = i - 1]; 
              i - 1
           ];

        Switch[typedegree,
           "mat", deg,
           "ent",
	            dc = Map[countzero,Transpose[pmc, {3, 1, 2}],{2}];
	            Map[If[#<0.,-Infinity,#]&,dc,{2}](*//Return*),
           "row",
	            dc = countzerovectors /@ Transpose[pmc];
	            Map[If[#<0.,-Infinity,#]&,dc](*//Return*),
           "col",
	            dc = countzerovectors /@ Transpose[pmc,{2, 3, 1}];
	            Map[If[#<0.,-Infinity,#]&,dc](*//Return*),
             _,
               Message[Deg::invopt]
        ]
    ];

(* scalar case *)
Deg[p:Poly[pc_,var_,deg_]] := deg;

(********************************************************) 
SquareQ[A:PolyMat[pmc_,var_,deg_]] := SameQ@@Dimensions[pmc[[1]]];
(* scalar case *)
SquareQ[pol:_Poly] := Message[Square::pol];
(********************************************************)

SylvestrMatrix[A:PolyMat[pmcA_,var_,deg_],col_?Positive] :=
   Module[{m, n, mat},
      {m,n} = Size[A];

      (* predpripravena matice ? 0 nebo 0. , ..  test vstupni matice Flatten[Map[Head, pmc, {3}]] // Union *)
      (* !! resit podobne jako v Det.m *)
      mat = Table[0,{m*(col+deg)},{col*n}];
      
	   Do[
	      If[ (i>=j && i<=j+deg), 
	         mat[[Range[(i-1)*m+1, i*m],Range[(j-1)*n+1, j*n]]] = pmcA[[i-j+1]]
	      ];
	      ,{i, deg+col}, {j, col}
	   ];
	   
	   ToPackedArray[mat]
   ]; 

(* Scalar case *)

SylvestrMatrix[p:Poly[pc_,var_,deg_],col_?Positive] :=
   Module[{mat},
      mat = Table[0,{col+deg},{col}];
      
      Do[
         If[(i >= j && i <= j + deg), mat[[i, j]] = pc[[i - j + 1]]]
            , {i, deg + col}, {j, col}
      ];
      
	   ToPackedArray[mat]
   ]; 



(******************************************************************************)
Star[p : PolyMat[pmc_, var_, deg_]] := 
PolyMat[ (Transpose /@ (pmc /. Complex[a_, b_] :> Complex[a, -b]))* Table[(-1)^i, {i, 0, deg}], var, deg];
(* Scalar case *)
(*Star[p:poly[pc,var,deg]] := Poly[Conjugate[pc]*Table[(-1)^i, {i, 0, deg}],var,deg];*)
(* !! jestlize konstanta je komplexni cislo -> chyba; asi lepsi |^*)
Star[p:Poly[pc_,var_,deg_]] := 
Poly[ (pc/.Complex[a_, b_] :> Complex[a, -b])*Table[(-1)^i, {i, 0, deg}],var,deg];
(******************************************************************************)

(******************************************************************************)
(* define CoefficientList[0,var] = {} and remove last zeros*)
CoefficientListZeroingLast[0|0., var_] := {};
CoefficientListZeroingLast[p_, var_] := With[{coe = CoefficientList[p,var]},
   If[ZeroQ[Last[coe]], DropLastZeros[coe], coe]];
CoefficientListZeroingLast[p:Poly[pc_,pvar_,deg_], pmvar_] := (
   If[pvar =!= pmvar, Message[General::difvars,pvar]];
   If[ZeroQ[Last[pc]], DropLastZeros[pc], pc]
)
(******************************************************************************)

DropLastZeros[dat_List] :=
   Module[{lendat = Length[dat], i},
   	i = lendat;
	  While[
   		ZeroQ[dat[[i]]],
	     	i = i - 1;
	  ];
	  Drop[dat, -(lendat - i)]
   ];
   
DropLastZerosPolMat[dat_] :=
   Module[{lendat = Length[dat], i},
   	i = lendat;
	  While[
   		And @@ Flatten[{ZeroQ[dat[[i]]]}],
	     	i = i - 1;
	  ];                      
	  Drop[dat, -(lendat - i)]	  
   ];
   
(*
DropLastZeros[dat_List] :=
   Module[{lendat = Length[dat], i, dropzer},
   	i = lendat;
	  While[
   		ZeroQ[dat[[i]]],
	     	i = i - 1;
	  ];
	  dropzer = Drop[dat, -(lendat - i)];
	  If[dropzer === List[], {0}, dropzer]
   ];
*)   
         

PrintVerbose[arg__] := 
   Print @@ (StyleForm[#, FontSize -> 10, FontFamily -> "Helvetica", 
              FontColor -> RGBColor[0.4, 0.2, 0.2]] & /@ {arg});
   
SymbolQ[expr_]:=AtomQ@Unevaluated[expr]&&Head@Unevaluated[expr]===Symbol;

NonZeroMin[data_] := Min[data /. (0|0.)->Infinity];



Size[pol:PolyMat[pmc_,var_,deg_]] := Dimensions[pmc[[1]]];
Size[pol:Poly[pc_,var_,deg_]] := {1,1};
   
End[]
    
EndPackage[]