BeginPackage["Polynomial`DESolve`","Polynomial`Object`","Polynomial`Common`"]
(* U S A G E *)

   DESolve::usage = " Solve Diophantine equations. ";
   AXBSolve::usage = " Solve A.X==B. "; 

(* O P T I O N S *)

(* M E S S A G E *) 

   General::varerr = " Different variables! ";
   AXBSolve::errdim = " Invalid size input matrices. Degree Ai = `1` and  Bi = `2` must be same. "   

Begin["`Private`"]
 
 
(********************************)
(*      MATRIX MANIPULATION     *)
(********************************)

SameColumnSize[l_List] := (SameQ @@ (Dimensions[#][[2]]& /@ l) )
SameRowSize[l_List] := (SameQ @@ (Dimensions[#][[1]]& /@ l) )

AppendColumns[l__?MatrixQ] := Join[l] /; SameColumnSize[{l}]

AppendRows[l__?MatrixQ] := Apply[Join, Transpose[{l}], {1}] /;
       SameRowSize[{l}]

ZeroMatrix[m_Integer,n_Integer] :=
        Table[0, {m}, {n}] /; m >= 0 && n>=0
         
 
(*****************************************************************)
(*             A.X == B                                          *)
(*****************************************************************)

AXBSolve[A_PolyMat, B_PolyMat] :=
Module[{i,
        degree,
        rdA, rdB,
        mindeg, maxdeg, rA, cA, rB, cB, degA, degB, rdeg, cdeg,
        mcA, c1A, sylA,
        mcB, c1B, sylB,
        listsol,
        sol,
        Xsol
       },

    {rA, cA} = Size[A]; degA = Deg[A];
    {rB, cB} = Size[B]; degB = Deg[B];
    rdA = Deg[A, TypeDegree -> "row"];
    rdB = Deg[B, TypeDegree -> "row"];

    If[rA =!= rB, Message[AXBSolve::errdim,rA,rB]; Return[Null]];

    (* Lower bound. *)
    mindeg = Max[Append[rdB - rdA, 0]];

    (* Upper bound. *)
    rdeg = Max /@ Transpose[{rdA, rdB}];
    rdeg = Join[Sort[rdeg, Greater], Table[0,{cA+cB-rA}]];
    cdeg = Join[Deg[A, TypeDegree -> "col"],Deg[B, TypeDegree -> "col"]];
    cdeg = Join[Sort[cdeg, Greater], Table[0,{rA-cA-cB}]];
    maxdeg = Min[Plus @@ rdeg[[Range[cA + cB - 1]]], Plus @@ cdeg[[Range[cA + cB - 1]]]];


    (* binary search for minimum degree *)
    (************************************)
    maxdeg = maxdeg + 1;
    mindeg = mindeg - 1;

    degree = mindeg + Floor[(maxdeg - mindeg)/2];
    listsol = {{}}; (* {} no solution *)

    mcA = A[[1]];
    mcB = B[[1]];


    While[maxdeg - 1 =!= mindeg,
        (* Sylvestr matrix A *)
        c1A = Flatten[mcA, 1]; (*AppendColumns*);
        c1A = Join[ c1A, ZeroMatrix[ degree*rA, cA ] ]; (* AppendColumns *)
        sylA = AppendRows @@ Table[RotateRight[c1A, i*rA], {i, 0, degree}];

        (* Sylvestr matrix B *)
        c1B = Flatten[mcB, 1]; (*AppendColumns*)
        sylB = Join[c1B, ZeroMatrix[(1+degA+degree)*rA - (1+degB)*rB, cB]]; (* AppendColumns *)

        (* solve equation *)
        Off[LinearSolve::"nosol"];
        Off[LinearSolve::"luc"];
        sol = LinearSolve[sylA, sylB];

        If[Head[sol] === LinearSolve,
            mindeg = degree,
            listsol = Append[listsol, sol]; maxdeg = degree
        ];

        degree = mindeg + Floor[(maxdeg - mindeg)/2];
    ];(* End While *)

    sol = Last[listsol];
    If[sol === {}, Return[{{}}]];
    Partition[sol, cA]
];

(*AXBSolve[aa_, ab_, avar___Symbol, {ax_}] := aXBSolve[aa, ab, $PM[[2]], {ax}] /;  (PMQ[aa, avar] && PMQ[ab, avar])*)


(*****************************************************************)
(*          Solve Diophantine equation                           *)
(*****************************************************************)
DESolve[eq_Equal] :=
Module[{
    eq1 = List @@ Subtract @@ eq,
    ax, xa, rs,
    lax, lxa,
    unkn, sizes, siz, foldsiz, sm, sn, maxdeg, idxs, A, axbsol, xysol, xsol,
    sp = False
   },
                                   (* zabudovat moznou jednotku *)
    ax = #[[1]]& /@ Cases[eq1, Dot[_PolyMat|_Poly, _Symbol]];
    xa = #[[2]]& /@ Cases[eq1, Dot[_Symbol, _PolyMat|_Poly]];
    rs = Cases[eq1, _PolyMat|_Poly]; (* !!! testovat zda je to pouze 1 matice*)
    lax = Length[ax];
    lxa = Length[xa];
   
    (* !! testovat promennou *) 
    
   If[Global`$Verbose,
      PrintVerbose["Verbose output for Diophantine equation a.x+b.y = c."];
      PrintVerbose[""];
   ];
       
    Which[
      lax > 0 && lxa === 0,
      (******************************************************************************)
         (* type axb *)
         
         (* skalarni polynomy *)
         ax = If[Head[#] === Poly, sp=True; PM[#], #]& /@ ax;
         rs = If[Head[#] === Poly, PM[#], #]& /@ rs;
         (***)
         
         unkn =  #[[2]]& /@ Cases[eq1, Dot[_PolyMat|_Poly, _Symbol]]; (* vylepsit spojenim s vypoctem ax *)
         sizes = Size /@ ax;
         If[Not[SameQ @@ (#[[1]]& /@ sizes)], Print["Message[General::err sizes]"]];
         sm = sizes[[1, 1]];
         maxdeg = Max[Deg /@ ax];
         
         siz = #[[2]]& /@ sizes;
         foldsiz = Drop[FoldList[Plus, 0, siz], 1];
         idxs = Apply[Range,
                      {{1, sm}, #}& /@ Transpose[{foldsiz - siz + 1, foldsiz}]
                      , {2}
               ];

         A = Table[0, {maxdeg + 1}, {sm}, {Plus @@ (#[[2]]& /@ sizes)}];
         
         Do[
            Do[
               A[[i, Sequence @@ idxs[[j]]]] = ax[[j, 1, i]]
               , {i, Deg[ax[[j]]] + 1}
            ]
            , {j, lax}
         ];

         axbsol = AXBSolve[PolyMat[A,ax[[1,2]],maxdeg], - rs[[1]]];

         idxs = Transpose[idxs][[2]];
         xysol = {};
         
         Do[
           AppendTo[xysol, unkn[[i]]->PolyMat[       (*!! osetrit nulovou matici !!*)
                                          xsol=DropLastZerosPolMat[(#[[idxs[[i]]]]& /@ axbsol)], ax[[1,2]], Length[xsol]-1
                                      ]
           ]
           ,{i, lax}
         ];

         If[sp,         
            xysol /. ((unkn_->PolyMat[pmc_, var_, deg_]) :> (unkn->Poly[Flatten[pmc], var, deg])),
            xysol
         ]
       ,
      lxa > 0 && lax === 0,
      (******************************************************************************)
         (* type xab *)
       ,
      True,
      (******************************************************************************)
         (* unknown type *)
         err[]
      ]
];

 
End[]
 
EndPackage[]