BeginPackage["Polynomial`Det`","Polynomial`Object`","Polynomial`Common`","Polynomial`Global`"]
(* U S A G E *)
(* O P T I O N S *) 
(* M E S S A G E *)
      
Begin["`Private`"]

Unprotect[PolyMat];

PolyMat /: Det[A:PolyMat[pmcA_,var_,deg_], opts___?OptionQ]  /; (
      SquareQ[A] && NumericQ[A] && Deg[A] > 0
        ) := 
Module[{ tol, me, prec,
        coe,
        coldegA, rowdegA, degofdet,
        samples, dets, sol,
        size = Dimensions[A][[2]]
       },

   coldegA = Deg[A,TypeDegree -> "col"];
   rowdegA = Deg[A,TypeDegree -> "row"];

   degofdet = Min[Plus @@ coldegA, Plus @@ rowdegA];

   (* zachovava precision *)
   prec = Min[Map[Precision, pmcA,{3}]];
   coe = Join[pmcA, 
            Array[
               Which[
                  prec === Infinity,
                  0&,
                  prec === $MachinePrecision,
                  0.&,
                  True,
                  SetAccuracy[0.,prec]& (*!! err - SetPrecision *)
               ],
               {degofdet-deg,size,size}
            ]
         ];

   samples =  Transpose[
               Map[ Fourier[#,FourierParameters -> {1, -1}]&, Transpose[coe,{3,1,2}],{2}]
             ,{2, 3, 1}];
   
   dets = Det /@ samples;
   
   (* zeroing *)                   
   tol = Global`$Zeroing;
   me = Min[Abs[NonZeros[Flatten[pmcA]]]];

   sol = DropLastZeros[    (* remove last zeros *)
            Chop[
               InverseFourier[
                  dets
                  ,FourierParameters -> {1, -1}
               ],
               (me * tol)/deg//N
            ]
         ];

   degofdet = Length[sol]-1;
   
   If[Global`$Verbose,
     PrintVerbose["Verbose output for Det[A], where A is a polynomial matrix."];
     PrintVerbose["Upper bound for degree of the determinant: \n", degofdet];
     PrintVerbose["Determinats: \n", dets];
   ];
     
   Poly[sol, var, degofdet]
];


PolyMat /: Det[A:PolyMat[pmcA_,var_,deg_], opts___?OptionQ] := 
Module[{coe},
   coe = Check[
      CoefficientList[Det[Pol[A]],var],
      Null
   ];
   If[ coe === Null,
      Null,
      Poly[coe,var,Length[coe]-1]
   ]
];
        
Protect[PolyMat];
     
End[]
 
EndPackage[]