(* G L O B A L    V A R I A B L E S *)
(* ******************************** *)
Global`$Verbose = True;
Global`$Format = "None";
Global`$Zeroing = 10.^-10;
Global`$Variable = Global`s;
Global`$ContinuousVariable = Global`s; 
Global`$DiscreteVariable = Global`z; 


BeginPackage["Polynomial`Global`","Polynomial`Object`","Polynomial`Common`"]

(* U S A G E *)
PolyGlobal::usage = " Set and print global variables. ";

(* O P T I O N S *)

(* M E S S A G E *)
PolyGlobal::errtype = " Unknown type for global variable: `1`. ";
PolyGlobal::unkvar = " Unknown global variable: `1`. ";


Begin["`Private`"]

(*        U S E F U L    F U N C T I O N S    *)
SymbolQ[expr_]:=AtomQ@Unevaluated[expr]&&Head@Unevaluated[expr]===Symbol;

(* $Format *)
FormatTypes = Alternatives["None","Nice","CoefTable","Pol"];


(* Print global variables *)
PolyGlobal[] := (
   Print["  $Format = ",Global`$Format];
   Print["  $Verbose = ",Global`$Verbose];
   Print["  $Zeroing = ",Global`$Zeroing];   
   Print["  $Variable = ",Global`$Variable];
   Print["  $ContinuousVariable = ",Global`$ContinuousVariable];
   Print["  $DiscreteVariable = ",Global`$DiscreteVariable];
);


(* Easy set global variables *)
PolyGlobal[arg_] := (
   If[Global`$Verbose, PrintVerbose["Verbose output for PolyGlobal[ ]."]];
   Switch[arg//N,
      FormatTypes, If[Global`$Verbose, PrintVerbose["  $Format = "<>ToString[arg]]];
                  Global`$Format = arg,
      True | False, If[Global`$Verbose, PrintVerbose["  $Verbose = "<>ToString[arg]]];
                  Global`$Verbose = arg,
      _Real | _Integer, If[Global`$Verbose, PrintVerbose["  $Zeroing = "<>ToString[InputForm[N[arg]]]]];
                  Global`$Zeroing = N[arg],
      _Symbol, If[Global`$Verbose, PrintVerbose["  $Variable = "<>ToString[arg]]];
                  Global`$Variable = arg,
      _, Message[PolyGlobal::errtype,arg]
   ]
);

(* Control input into variables *)

(* $Format *)
FormatTypes = Alternatives["None","Nice","CoefTable","Pol"];

SetFormat = True;
Set[Global`$Format, type_] /; SetFormat ^:= (
   SetFormat = False;
   If[MatchQ[type,FormatTypes],
      LastFormat = Global`$Format; (* for Format[]=. in PolyFormat[]*)
      Global`$Format = type;
      SetFormat = True; PolyFormat[type]; Return[type],
      Message[PolyGlobal::errtype,type];
      Global`$Format = Global`$Format; (*last value*)
      SetFormat = True; Return[Null]
   ]
);

(* $Verbose *)
SetVerbose = True;
Set[Global`$Verbose, boo_] /; SetVerbose ^:= (
   SetVerbose = False;
   If[(boo === True || boo === False),
      Global`$Verbose = boo;
      SetVerbose = True; Return[boo],
      Global`$Verbose = Global`$Verbose; (*last value*)
      Message[PolyGlobal::errtype,boo];
      SetVerbose = True; Return[Null]
   ]
);

(* $Zeroing *)
SetZeroing = True;
Set[Global`$Zeroing, num_] /; SetZeroing ^:= (
   SetZeroing = False;
   If[NumericQ[num],
      Global`$Zeroing = N[num];
      SetZeroing = True; Return[N[num]],
      Global`$Zeroing = Global`$Zeroing; (*last value*)
      Message[PolyGlobal::errtype,num];
      SetZeroing = True; Return[Null]
   ]
);

(* $Variable *)
SetVariable = True;
Set[Global`$Variable, var_] /; SetVariable ^:= (
   SetVariable = False; 
   If[SymbolQ[var],
      Global`$Variable = var;
      SetVariable = True; Return[var],
      Message[PolyGlobal::errtype,var];
      Global`$Variable = Global`$Variable; (*last value*)
      SetVariable = True; Return[Null]
   ]
);

(* $ContinuousVariable *)
Set[Global`$DiscreteVariable, var_] /; SetVariable ^:= (
   SetVariable = False; 
   If[SymbolQ[var],
      Global`$ContinuousVariable = var;
      SetVariable = True; Return[var],
      Message[PolyGlobal::errtype,var];
      Global`$ContinuousVariable = Global`$ContinuousVariable; (*last value*)
      SetVariable = True; Return[Null]
   ]
);

(* $DiscreteVariable *)
Set[Global`$DiscreteVariable, var_] /; SetVariable ^:= (
   SetVariable = False; 
   If[SymbolQ[var],
      Global`$DiscreteVariable = var;
      SetVariable = True; Return[var],
      Message[PolyGlobal::errtype,var];
      Global`$DiscreteVariable = Global`$DiscreteVariable; (*last value*)
      SetVariable = True; Return[Null]
   ]
);


(********************************************************************)
(*     Poly Format Support                                          *)
(********************************************************************)
PolyFormat[type_] := (
   Unprotect /@ {PolyMat,Poly};
   Switch[type,
      "Nice",
            Format[PolyMat[pmc_,var_,deg_]] := Subscripted[
               MatrixForm[
                  If[deg>=0, Plus @@ Table[pmc[[i + 1]]*var^i, {i, 0, deg}], pmc[[1]]]
               ][var]];
            (* SCALAR CASE *)      
            Format[Poly[pc_,var_,deg_]] := Subscripted[
               MatrixForm[
                  If[deg>0, {Plus @@ Table[pc[[i + 1]]*var^i, {i, 0, deg}]}, {pc}]
               ][var]],
      "CoefTable",
         Format[PolyMat[pmc_,var_,deg_]] := 
            With[{dim = Dimensions[pmc],
                  style = StyleForm[#, FontSize -> 10, FontFamily -> "Helvetica", FontColor -> RGBColor[0, 0, 1]]&
                 },
  	            TableForm[pmc, 
  	               TableHeadings -> Map[style,{Range[0,dim[[1]]-1], Range[1,dim[[2]]], Range[1, dim[[3]]]},{2}],
  	               TableSpacing -> {3, 1, 1, 3}
               ]
  	         ];
         (* SCALAR CASE *)            
         Format[Poly[pc_,var_,deg_]] := 
            TableForm[pc,TableDirections->{Row}, 
                         TableHeadings->{StyleForm[#, FontSize->10, FontFamily -> "Helvetica", FontColor -> RGBColor[0, 0, 1]]& /@ Range[0,deg]}
            ],
      "None",
         Format[PolyMat[pmc_,var_,deg_]] := HoldForm[PMC[pmc,var,deg]]; (* !! pozor HoldForm se prenasi kopirovanim *)
         (* SCALAR CASE *)            
         Format[Poly[pc_,var_,deg_]] := HoldForm[PC[pc,var,deg]],
      "Pol",
         If[ LastFormat =!= "Pol", Format[PolyMat[pmc_,var_,deg_]] =.];
         (* SCALAR CASE *)            
         If[ LastFormat =!= "Pol", Format[Poly[pc_,var_,deg_]] =.]
         
   ];
   Protect /@ {PolyMat,Poly}
);

(* inicialization *)
PolyFormat[Global`$Format];

End[ ]

EndPackage[]