(* :Title:
   POLYNOMIAL TOOLBOX - Data Types Definitions and Conversions*)
 
(* :Context:
   Polynomial`Object`  *)
 
(* :Author:
   Petr Kujan, Martin Hromcik  *)
  
 
BeginPackage["Polynomial`Object`","Polynomial`Common`"]

(* U S A G E *)
   (* POLYNOMIAL MATRIX *)
   PM::usage = " Polynomial matrix. ";
   PMC::usage = " Matrix coefficients of polynomial matrix. ";
   PLC::usage = " Matrix of lists coefficients of polynomial matrix. ";
   (* SCALAR POLYNOM *)
   P::usage = " Scalar polynom. ";
   PC::usage = " Coefficients of scalar polynom. ";

(* O P T I O N S *)

(* M E S S A G E *)
    General::notpoly = " The matrix `1` not polynomial. ";
    General::notpolmat = " Intput `1` can not by used as PM. ";
    General::notpolmatc = " Intput list of coefficients `1` can not contain variable `2`. ";
    General::notpollistc = " Intput matrix of coefficients `1` can not contain variable `2`. ";
    General::notpol = " Intput `1` is not polynom. ";
    General::badarg = " The matrix `1` has more than one variable or not polynomial. Expect argument for variable. ";
    General::isnotsymbol = " Input variable `1` is not Symbol. Expect symbol. ";
    General::badargvar = " More input arguments as variable: `1`. Expect one or nothing. ";
    General::notcoelist = " Matrices of input list has not same size. ";
    General::notlistcoe = " Matrix of coefficient lists is invalid. ";
    General::notpolyformat = " Input is not in Poly format. ";
    General::badvar = " Input variable `1` is in polynom as parametr. ";    
    General::errcoe = " Input coefficients contain variable `1`. ";
    General::errcoe1 = " Input coefficients contain variable which is polynomial object. "; 


Begin["`Private`"]
(* developer context *)
   <<Developer`

(******************************************************************************)
(*          U S E F U L    F U N C T I O N S                                  *)
(******************************************************************************)
(* matrices coefficients of polynomial matrix are only matrices of same size *) 
PMCoefficientsQ[list_List] :=
If[MatrixQ[First[list]] && (SameQ @@ (Dimensions /@ list)), True,
   Message[General::notcoelist]; False
]; (* PMC *)

(* lists coefficients of polynomial matrix are only one matrix with lists *)
PLCoefficientsQ[mat_List] :=
If[((And @@ Flatten[Map[VectorQ[#]&, mat, {2}]]) && 
   ((Length[Dimensions[mat]]===2)||(Length[Dimensions[mat]]===3))), True, 
    Message[General::notlistcoe]; False
]; (* PLC *)

(* this defines SymbolQ that tests whether or not expr is a symbol. *)
Attributes[SymbolQ] = {HoldAllComplete};
SymbolQ[expr_] := AtomQ[Unevaluated[expr]] && Head[Unevaluated[expr]] === Symbol;

(* Return variable or ErrVar[]*)
GetVariable[A_] :=
With[{fvar = Variables[A]},
  (* find variable *)
  If[Length[fvar] > 1,
      Message[General::badarg, Short[A]]; ErrVar[],
      If[fvar === {}, 
         Global`$Variable,
         If[Head[fvar[[1]]]===Symbol,
            fvar[[1]],
            Message[General::isnotsymbol,fvar[[1]]]; ErrVar[]
         ]
      ]
  ]
];


(******************************************************************************)
(*          P O L Y N O M I A L   P A T T E R N                               *)
(******************************************************************************)
Off[Rule::rhs];
(* POLYNOMIAL MATRIX *)
polymat[pmc_,var_,deg_] =  PolyMat[pmc_, var_, deg_];
polymat[] =  PolyMat[_?PMCoefficientsQ, _Symbol, _?PositiveQ]; (* Plus, .. *)

(* SCALAR POLYNOM *)
poly[pc_,var_,deg_] = Poly[pc_, var_, deg_];
On[Rule::rhs];


polyobj = (poly[pc,var,deg] | polymat[pmc,var,deg]);

(******************************************************************************)
(*          PM                                                                *)
(******************************************************************************)
PM[pm_?MatrixQ, var_Symbol] /; PMTest[pm,var] :=
Module[{plc = Map[CoefficientListZeroingLast[#, var]&, pm, {2}],
      maxdeg},
   maxdeg = Max[Map[Length, plc, {2}]];
   If[maxdeg =!= 0,
      PolyMat[
         Transpose[
               Map[PadRight[#, maxdeg, 0]&, plc, {2}], (* !! doplnit 0. nebo 0 ?? -> !paq *)
               {2,3,1}
            ]//ToPackedArray,
         var, maxdeg-1
      ],
      PolyMat[
         Array[0&, Dimensions[pm]]//ToPackedArray,
         var, -Infinity
      ]
   ]
];

(** another possible definitions **)
PM[pm_?MatrixQ] := PM[pm, GetVariable[pm]];

PM[pm_?MatrixQ, _ErrVar] := Null;

(** conversion **)
PM[pol_PolyMat] := pol;

PM[pol:Poly[pc_,var_,deg_]] := PolyMat[{{#}}& /@ pc, var, deg];

(** other definition is Null **)
PM[___] := (Message[General::notpolyformat]; Null);

(******************************************************************************)
(*          PMC                                                               *)
(******************************************************************************)
PMC[pmc_?PMCoefficientsQ, var_Symbol:Global`$Variable] /; PMCTest[pmc,var] :=
Module[{sol},
   If[And @@ Flatten[ZeroQ[Last[pmc]]], 
      sol = DropLastZerosPolMat[pmc];
      If[sol =!= {},
         PolyMat[sol//ToPackedArray, var, Length[sol]-1],
         PolyMat[Array[0&,Dimensions[pmc[[1]]]]//ToPackedArray, var, -Infinity]
      ],
      PolyMat[pmc//ToPackedArray,var,Length[pmc]-1]
   ]     
];

(** conversion **)
PMC[pol_PolyMat] := pol;

PMC[pol:Poly[pc_,var_,deg_]] := PolyMat[{{#}}& /@ pc, var, deg];

(** other definition is Null **)
PMC[___] := (Message[General::notpolyformat]; Null);

(******************************************************************************)
(*          PLC                                                               *)
(******************************************************************************)
PLC[plc_?PLCoefficientsQ, var:_Symbol:Global`$Variable] /; PLCTest[plc,var] :=   
With[{maxdeg = Max[Map[Length[DropLastZeros[#]]&, plc, {2}]]},
   If[maxdeg =!= 0,
      PolyMat[
         Transpose[
            Map[PadRight[#, maxdeg, 0] &, plc, {2}] (* !! doplnit 0. nebo 0 ?? -> !paq *)
	        ,{2, 3, 1}
         ]//ToPackedArray, var, maxdeg-1
      ],
      PolyMat[Array[0&, Dimensions[plc]], var, -Infinity]
   ]
];   

(** conversion **)
PLC[pol_PolyMat] := pol;

PLC[pol:Poly[pc_,var_,deg_]] := PolyMat[{{#}}& /@ pc, var, deg];

(** other definition is Null **)
PLC[___] := (Message[General::notpolyformat]; Null);

(* SCALAR CASE *)
(******************************************************************************)
(*          P                                                                 *)
(******************************************************************************)
P[p_, var_Symbol] /; PTest[p,var] := 
With[{pc = CoefficientListZeroingLast[p,var]},
   If[ Cases[pc,_?PolyQ,{0,Infinity}]=!={},
         Message[General::errcoe1]; Return[Null] 
   ];

   If[Length[pc] =!= 0,
         Poly[ToPackedArray[pc], var, Length[pc]-1],
         Poly[{0}, var, -Infinity]
   ]
];

(** another possible definitions **)
P[a_Symbol?(#=!=Global`$Variable&)] := Poly[{a}, Global`$Variable,0];

P[p_] := P[p, GetVariable[p]];

P[p_, _ErrVar] := Null;

(** conversion **)
P[pol_Poly] := pol;

(** other definition is Null **)
P[___] := (Message[General::notpolyformat]; Null);

(******************************************************************************)
(*          PC                                                                *)
(******************************************************************************)
PC[pc_?VectorQ, var_Symbol:Global`$Variable] /; PCTest[pc,var] := 
With[{pczer = DropLastZeros[pc]},
   If[ Cases[pc,_?PolyQ,{0,Infinity}]=!={},
         Message[General::errcoe1]; Return[Null] 
   ];
   
   If[pczer =!= {},
      Poly[pczer//ToPackedArray, var, Length[pczer]-1],
      Poly[{0}, var, -Infinity]
   ]
]

(** another possible definitions **)            
PC[pc_?(Not[PolyQ[#]] && NumericQ[#]&), var_Symbol:Global`$Variable] := PC[{pc}, var];

PC[pc_Symbol, var_Symbol:Global`$Variable] := PC[{pc}, var, 0];

(** conversion **)
PC[pol_Poly] := pol;

(** other definition is Nul **)
PC[___] := (Message[General::notpolyformat]; Null);      (* one or more bad arguments *)


(**  P O L Y   O B J E C T  -  Poly[][]  **)
PolyMat[pc_,var_,deg_][newvar_Symbol] := 
   If[PackedArrayQ[pc], (* no symbols *)
      PolyMat[pc,newvar,deg],
      If[Intersection[Variables[pc],{newvar}]==={},
         PolyMat[pc,newvar,deg],
         Message[General::badvar, newvar];
         Null
      ]
   ];

PolyMat[pc_,var_,deg_][val_?(Not[PolyQ[#]] && NumericQ[#]&)] :=     (* Plus @@ Table[pc[[i + 1]]*val^i, {i, 0, deg}] *) (* slower *)
   With[{pow = ToPackedArray[Array[val^#&, {deg+1}, 0]]},
      Inner[Times, pc, pow, Plus,1]
   ];

(* SCALAR CASE *)
Poly[pc_,var_,deg_][newvar_Symbol] :=
   If[PackedArrayQ[pc],
      Poly[pc,newvar,deg],
      If[Intersection[Variables[pc],{newvar}]==={},
         Poly[pc,newvar,deg],
         Message[General::badvar, newvar];
         Null
      ]
   ];
   
Poly[pc_,var_,deg_][val_?(Not[PolyQ[#]] && NumericQ[#]&)] := PolyMat[pc, var, deg][val];


(******************************************************************************)
(*          T E S T S                                                         *)
(******************************************************************************)
(* PMTest - test if input matrix mat is polynomial in var *)
PMTest[mat_,var_] := (             (* !! upravit podle PP str. 213 - SetAttr..*)
   If[Global`$Verbose, PrintVerbose["Tests if input is PM[]."]];
   If[(And @@ Flatten[ Map[ (PolynomialQ[#, var] || MatchQ[#, _Poly])&, mat, {2}] ]) (* ||
      (And @@ Flatten[ PolynomialQ[Shift[m, v] , v]])
      *),
      True,
      Message[General::notpoly, Short[mat]];
      False
   ]
);

(* PMCTest - tests if coefficients don't contain variable as parametr *)
PMCTest[coe_,var_] := (
      If[Global`$Verbose, PrintVerbose["Tests if input is PMC[]."]];
      If[Intersection[Variables[coe],{var}] === {var},
         Message[General::errcoe,var]; False,
         True
      ]
);

(* PLCTest - tests if coefficients don't contain variable as parametr *)
PLCTest[plc_,var_] := (
      If[Global`$Verbose, PrintVerbose["Tests if input is PLC[]."]];
      If[Intersection[Variables[plc],{var}] === {var},
         Message[General::errcoe,var]; False,
         True
      ]  (* !! doplnit Messages[] *)      
);

(* SCALAR CASE *)
(* PTest - tests if input is polynom *)
PTest[p_,var_] := (
      If[Global`$Verbose, PrintVerbose["Tests if input is P[]."]];
      If[PolynomialQ[p, var], True, Message[General::notpol,p]; False]
      (* funkce PolynomialQ[] je velmi pomala; napr. pro deg=10^3  -> casova nar. = 6 sec*)
);
    
(* PCTest - tests if input list is vector of coefficients *)
PCTest[coe_, var_] := (
      If[Global`$Verbose,PrintVerbose["Tests if input is PC[]."]];
      If[Intersection[Variables[coe],{var}] === {var},Message[General::errcoe,var];False,True]
);

    
(***************************************)

End[ ]
   Protect[PolyMat];
   Protect[Poly];
EndPackage[]