BeginPackage["Polynomial`PlotStability`",
             "Graphics`InequalityGraphics`","Graphics`ImplicitPlot`",
             "Utilities`FilterOptions`",
             "Polynomial`Object`","Polynomial`Common`"]
(* U S A G E *)
RouthTable::usage = " Return Routh's table. ";
DRouthTable::usage = " Return discrete Routh's table. ";
JuryTable::usage = " Discrete  time - Routh's table: Jury - Marden. ";
StabilityPlot::usage = " Plot stability area. ";
StabilityPlot3D::usage = " Plot stability area. ";
TStabilityPlot::usage = " Plot stable areas for first column of Routh's table. ";

(* O P T I O N S *)
Options[RouthTable] = {ZeroSymbol->Global`zero, SimplifyFunction->Automatic}
Options[JuryTable] = {SimplifyFunction->Automatic}

Options[StabilityPlot] = Join[{PlotArea->Automatic, DefaultRange->{{-50,50},{-50,50}}},
                                 Options[InequalityPlot], Options[ImplicitPlot]
                         ];

Options[StabilityPlot3D] = Join[{DefaultRange->{{-50,50},{-50,50},{-50,50}}},
                                 Options[InequalityPlot3D]
                         ];

(* M E S S A G E *) 
RouthTable::unknopt = " Unknown value for option SimplifyFunction. ";
DRouthTable::unknopt = " Unknown value for option SimplifyFunction. ";
JuryTable::unknopt = " Unknown value for option SimplifyFunction. ";
StabilityPlot::errvars = " Error parameters. Two parametrs are expected. ";
StabilityPlot::errpar = " More parametrs. "
StabilityPlot::unkopt = " Unknown value for option PlotArea. ";
StabilityPlot::unboudedstabarea = " Unboded stable region, use default range `1`. ";
StabilityPlot::usecon = " Use contour plot. ";

Begin["`Private`"]

(******************************************************************************)
(*          Useful functions                                                  *)
(******************************************************************************)
Vars[pol:Poly[pc,var,deg]] := Union[Cases[Variables[pc], _Symbol, Infinity]];

(******************************************************************************)
(*          Routh's Table                                                     *)
(******************************************************************************)
RouthTable[chp:Poly[pc_,var_,deg_], opts___?OptionQ] := 
Module[{ routab,
         zer, const, simplify,
         coefs = Reverse[pc],
         iszer = False,
         me = Min[Abs[NonZeros[pc]]],
         optZeroSym = ZeroSymbol /. {opts} /. Options[RouthTable],
         optSimplifyFunc = SimplifyFunction /. {opts} /. Options[RouthTable]
       }, 

(** Simlify options ***********************************************************)
   Switch[ optSimplifyFunc,
      Automatic,
         If[ NumericQ[chp],
            simplify = #&, (* do nothing *)
            simplify = ExpandNumerator[ExpandDenominator[Together[#]]]&;
         ],
      None, 
         simplify = #&,
      Simplify, 
         simplify = Simplify[#]&,
      FullSimplify, 
         simplify = FullSimplify[#]&,
      _,
         Message[RouthTable::unknopt];
         simplify = #&
   ];
(******************************************************************************)
    
   routab = {};
   AppendTo[routab, Take[coefs, {1, deg+1, 2}]];
   AppendTo[routab, Take[coefs, {2, deg+1, 2}]];
   
   If[Abs[routab[[2,1]]] < Global`$Zeroing*me, zer = routab[[2,1]]; routab[[2,1]]=optZeroSym; iszer = True];
   
   If[EvenQ[deg],
     (* then *)
      const = Last[routab[[1]]];
      Do[
         row = 
         If[OddQ[i],
            Append[
               (routab[[i + 1, 1]]*Drop[Rest[routab[[i]]], -1] - routab[[i, 1]]*Rest[routab[[i + 1]]])/routab[[i + 1, 1]],
               const
            ],
            (routab[[i + 1, 1]]*Rest[routab[[i]]] - routab[[i, 1]]*Rest[routab[[i + 1]]])/routab[[i + 1, 1]]
         ];
         If[Abs[row[[1]]] < Global`$Zeroing*me, zer = Abs[row[[1]]]; row[[1]] = optZeroSym; iszer = True ];
         AppendTo[routab, simplify[row]];
         ,{i, 1, deg - 1}
      ],
      (* else *)
      const = Last[routab[[2]]];
      Do[
         row = 
         If[OddQ[i],
             (routab[[i + 1, 1]]*Rest[routab[[i]]] - routab[[i, 1]]*Rest[routab[[i + 1]]])/routab[[i + 1, 1]],
              Append[
                 (routab[[i + 1, 1]]*Drop[Rest[routab[[i]]], -1] - routab[[i, 1]]*Rest[routab[[i + 1]]])/routab[[i + 1, 1]],
                 const
              ]
         ];
         If[Abs[row[[1]]] < Global`$Zeroing*me, zer = row[[1]]; row[[1]] = optZeroSym; iszer = True];
         AppendTo[routab, simplify[row]];
         ,{i, 1, deg - 1}
      ]
   ];

(** Verbose output ************************************************************)
   If[Global`$Verbose && iszer,
      PrintVerbose["Verbose output for RouthTable:"];
      PrintVerbose["Zero:  ", zer];
      PrintVerbose[ExpandNumerator[ExpandDenominator[Together[routab]]]//TableForm]
   ];
(******************************************************************************)

   If[iszer, 
      routab = (Limit[#, optZeroSym -> 0, Direction -> -1]& /@ routab
         (*ExpandNumerator[ExpandDenominator[Together[routab]]]*))
   ];

   routab
];

(******************************************************************************)
(*          Discrete time - Jury-Marden                                       *)
(******************************************************************************)
JuryTable[chp:Poly[pc_,var_,deg_], opts___?OptionQ] := 
Module[{ jurytab,
         simplify, const1, const2,
         me = Min[Abs[NonZeros[pc]]],
         optSimplifyFunc = SimplifyFunction /. {opts} /. Options[JuryTable]
       }, 
   
(** Simlify options ***********************************************************)
   Switch[ optSimplifyFunc,
      Automatic,
         If[ NumericQ[chp],
            simplify = #&, (* do nothing *)
            simplify = Expand[#]&;
         ],
      None, 
         simplify = #&,
      Simplify, 
         simplify = Simplify[#]&,
      FullSimplify, 
         simplify = FullSimplify[#]&,
      _,
         Message[JuryTable::unknopt];
         simplify = #&
   ];
(******************************************************************************) 
    
   jurytab = {};
   AppendTo[jurytab, Reverse[pc]];
   AppendTo[jurytab, pc];

   Do[
      row = 
      If[ OddQ[i],
         const1 = First[jurytab[[i-2]]];
         const2 = First[jurytab[[i-1]]];
         const1*Rest[jurytab[[i-1]]] - const2*Rest[jurytab[[i-2]]]
         ,
         Reverse[jurytab[[i-1]]]
      ];
      AppendTo[jurytab, simplify[row]];
      ,{i, 3, 2*(deg+1)}
   ];

   jurytab
];


(******************************************************************************)
(*          Discrete Routh's table                                            *)
(******************************************************************************)
(* Teorie automatickeho rizeni - Priklady, str. 140 *)
DRouthTable[chp:Poly[pc_,var_,deg_], opts___?OptionQ] := 
Module[{ droutab,
         const1, const2, simplify, zer,
         iszer = False,
         me = Min[Abs[NonZeros[pc]]],
         optZeroSym = ZeroSymbol /. {opts} /. Options[JuryTable],
         optSimplifyFunc = SimplifyFunction /. {opts} /. Options[JuryTable]
       }, 
   
(** Simlify options ***********************************************************)
   Switch[ optSimplifyFunc,
      Automatic,
         If[ NumericQ[chp],
            simplify = #&, (* do nothing *)
            simplify = ExpandNumerator[ExpandDenominator[Together[#]]]&;
         ],
      None, 
         simplify = #&,
      Simplify, 
         simplify = Simplify[#]&,
      FullSimplify, 
         simplify = FullSimplify[#]&,
      _,
         Message[DRouthTable::unknopt];
         simplify = #&
   ];
(******************************************************************************) 
    
   droutab = {};
   AppendTo[droutab, Reverse[pc]];
   AppendTo[droutab, pc];
   
   If[Abs[droutab[[1,1]]] < Global`$Zeroing*me, zer = droutab[[2,1]]; droutab[[2,1]]=optZeroSym; iszer = True];
   
   Do[
      If[ OddQ[i],
         const1 = First[droutab[[i-1]]];
         const2 = First[droutab[[i-2]]];
         row = Drop[- const1/const2*droutab[[i-1]] + droutab[[i-2]],-1];
         If[Abs[row[[1]]] < Global`$Zeroing*me, zer = row[[1]]; row[[1]] = optZeroSym; iszer = True ]
         ,
         row = Reverse[droutab[[i-1]]]
      ];
      AppendTo[droutab, simplify[row]];
      ,{i, 3, 2*(deg+1)}
   ];

(** Verbose output ************************************************************)
   If[Global`$Verbose && iszer,
      PrintVerbose["Verbose output for RouthTable:"];
      PrintVerbose["Zero:  ", zer];
      PrintVerbose[ExpandNumerator[ExpandDenominator[Together[droutab]]]//TableForm]
   ];
(******************************************************************************)
 
   If[iszer, 
      droutab = (Limit[#, optZeroSym -> 0, Direction -> -1]& /@ droutab
         (*ExpandNumerator[ExpandDenominator[Together[droutab]]]*))
   ];

   droutab
];


(******************************************************************************)
(*          Stability plot                                                    *)
(******************************************************************************)
<<Graphics`InequalityGraphics`;
<<Graphics`ImplicitPlot`;

StabilityPlot[charpoly:Poly[pc_,var_,deg_], {p1_,lp1_,rp1_} ,{p2_,lp2_,rp2_}, opts___?OptionQ] :=
Module[{ fcol, ineqs,
         vars = Vars[charpoly],
         optPlotArea = PlotArea /. {opts} /. Options[StabilityPlot],
         plotStableArea, plotBorders,
         graphics, gdata
       },

   If[ Length[vars]>2, Message[StabilityPlot::errpar]; Return[Null]];

(** Continuous or Discrete time ***********************************************)
   Switch[var, 
   (** Continuous time ********************************************************)     
      Global`$ContinuousVariable,
         (* Routh's table *)
         fcol = First /@ RouthTable[charpoly];
         
         (* If zero is in the first column Routh's table and Limit[] returns DirectedInfinity[] *)
         If[
            Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
            fcol = (fcol /. DirectedInfinity[arg_] -> arg);
         ];
         
         (* Inequalities *)
         ineqs = # >= 0& /@ fcol
      ,   
   (** Discrete time **********************************************************)
      Global`$DiscreteVariable,
      (*
         (* Jury's table *)
         fcol = First /@ Take[JuryTable[charpoly], {4, 2(deg + 1), 2}];
         (* Inequalities *)
         ineqs = Prepend[(# < 0)& /@ Rest[fcol], First[fcol] > 0]
      *)
         (* DRouthTable's table *)
         fcol = First /@ Take[DRouthTable[charpoly], {1, 2(deg + 1), 2}];
         
         (* Inequalities *)
         ineqs = (# >= 0)& /@ fcol
      ,
   (** Continuous time ********************************************************)     
      _,
         fcol = First /@ RouthTable[charpoly];
         If[
            Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
            fcol = (fcol /. DirectedInfinity[arg_] -> arg);
         ];
         ineqs = (# >= 0)& /@ fcol
   ]; (* End Switch *)
(******************************************************************************)
        
(** Plot stable area **********************************************************)
   If[
      Check[
         (* plot stable area *)
         plotStableArea = InequalityPlot[
            And @@ ineqs, {p1,lp1,rp1}, {p2,lp2,rp2},
            DisplayFunction -> Identity, 
            FilterOptions[InequalityPlot,opts],
            PlotRange->All, Frame -> False, Axes -> False
         ],
         region, InequalityPlot::"region"
      ] === region,
      (* then *)
      Message[StabilityPlot::usecon];
      If[ Union[Cases[ineqs, False]] === {False},
         plotStableArea = ContourPlot[0, {p1,lp1,rp1}, {p2,lp2,rp2}, (* empty graphics *)
            DisplayFunction -> Identity, ContourSmoothing -> False, Contours -> 1,
            PlotRange -> All, ColorFunction -> (If[# > 0, GrayLevel[1], Hue[.5]]&),
            Frame -> False, Axes -> False
         ],
         plotStableArea = 
         ContourPlot[ If[And @@ ((Sign[Denominator[#]]*Numerator[#] >= 0)& /@ fcol), 0, 1]//Evaluate, {p1,lp1,rp1}, {p2,lp2,rp2},
            DisplayFunction -> Identity, 
            ContourSmoothing -> False, Contours -> 1,
            FilterOptions[ContourPlot,opts]//Evaluate,
            PlotPoints -> {200, 200},
            PlotRange -> All,
            ColorFunction -> (If[# > 0, GrayLevel[1], Hue[.5]]&),
            Frame -> False, Axes -> False
         ]
      ]
   ];
(******************************************************************************)

(** Options PlotArea **********************************************************)
   Switch[ optPlotArea,
   (** Stable plot area *******************************************************)   
      Automatic,  (** plot only stable area **)
         graphics = plotStableArea,

   (** Plot borders ***********************************************************)
      Global`Border,  (** plot stable area with unstable borders **)
         gdata = Cases[Numerator[#] == 0& /@ fcol, _Equal];

         plotBorders = 
            If[gdata =!= {},
               ImplicitPlot[
                  gdata, {p1,lp1,rp1}, {p2,lp2,rp2},
                  DisplayFunction -> Identity, PlotRange -> All, Frame -> True, Axes -> True,
                  FilterOptions[ImplicitPlot,opts],
                  PlotStyle->{Hue[.7]}
               ]
            ];

         graphics = {     
            plotStableArea,  (* plot stable area *)
            FullGraphics[plotBorders]  (* plot borders *)                                    
         },
   (** Unknown value for option ***********************************************)               
      _,
         Message[StabilityPlot::unkopt];
         graphics = plotStableArea
   ]; (* End Switch *)  
(******************************************************************************)

   Show[ graphics,
      DisplayFunction -> $DisplayFunction,
      FilterOptions[Graphics,opts],
      (*Axes -> True,*)
      AxesLabel -> {ToString[p1],ToString[p2]}
   ]
];


StabilityPlot[charpoly:Poly[pc_,var_,deg_], opts___?OptionQ ] := 
Module[{ fcol,
         vars = Union[Cases[Variables[pc], _Symbol, Infinity]],
         optDefaultRange = DefaultRange /. {opts} /. Options[StabilityPlot],
         optPlotArea = PlotArea /. {opts} /. Options[StabilityPlot],
         graphics
       },
    
   If[ Length[vars] > 2, Message[StabilityPlot::errvars];Return[Null]];

   (* Tests if stability area is bounded *)
   If[ optPlotArea =!= Automatic,
      (* then *)
      StabilityPlot[ charpoly,
                     {vars[[1]],optDefaultRange[[1,1]],optDefaultRange[[1,2]]},
                     {vars[[2]],optDefaultRange[[2,1]],optDefaultRange[[2,2]]},
                     opts         
      ],
      (* else *)      
      fcol = First/@RouthTable[charpoly];
    
      (* If zero is in the first column Routh's table and Limit[] returns DirectedInfinity[] *)
      If[
         Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
         fcol = (fcol /. DirectedInfinity[arg_] -> arg);
      ];
      
      If[
         Check[
            (* plot stable area *)
            graphics = InequalityPlot[
               And @@ ((# >= 0) & /@ fcol), {vars[[1]]}, {vars[[2]]},
               DisplayFunction -> Identity, 
               FilterOptions[InequalityPlot,opts],
               PlotRange->All, AxesLabel->{ToString[vars[[1]]],ToString[vars[[2]]]}
            ],
            err, InequalityPlot::"unbounded", InequalityPlot::"region"
         ] === err,
         (* then *)
         Message[StabilityPlot::unboudedstabarea,optDefaultRange]; (* !! upravit message? *)
         StabilityPlot[ charpoly,
                        {vars[[1]],optDefaultRange[[1,1]],optDefaultRange[[1,2]]},
                        {vars[[2]],optDefaultRange[[2,1]],optDefaultRange[[2,2]]},
                        opts          
         ],
         (* else *)
         Show[graphics, DisplayFunction->$DisplayFunction, FilterOptions[Graphics,opts]]
      ]
   ]
];

(******************************************************************************)
(*          Stability plot 3D                                                 *)
(******************************************************************************)
(* !! dodelat: spojity x diskretni *)
StabilityPlot3D[charpoly:Poly[pc_,var_,deg_], {p1_,lp1_,rp1_}, {p2_,lp2_,rp2_}, {p3_,lp3_,rp3_}, opts___?OptionQ] :=
Module[{ fcol, zer,
         vars = Union[Cases[Variables[pc], _Symbol, Infinity]]
       },
      
   If[Length[vars] > 3, Message[StabilityPlot::errpar]; Return[Null]];
   
   fcol = First /@ RouthTable[charpoly];

   (* if zero is in the first column Routh's table and Limit[] returns DirectedInfinity[] *)
   If[
      Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
      fcol = (fcol /. DirectedInfinity[arg_] -> arg);
   ];
   
   InequalityPlot3D[  (** plot stable area **)
      And @@ ((# >= 0) & /@ fcol), {p1,lp1,rp1}, {p2,lp2,rp2}, {p3,lp3,rp3},
      FilterOptions[InequalityPlot,opts],
      PlotRange->All, AxesLabel -> {ToString[p1],ToString[p2],ToString[p3]}
   ]
];

StabilityPlot3D[charpoly:Poly[pc_,var_,deg_], opts___?OptionQ ] := 
Module[{ vars = Union[Cases[Variables[pc], _Symbol, Infinity]],
         optDefaultRange = DefaultRange /. {opts} /. Options[StabilityPlot3D],
         graphics
       },
      
   If[Length[vars] > 3, 
      Message[StabilityPlot::errvars]; Return[Null]
   ];

   fcol = First /@ RouthTable[charpoly];
 
   (* if zero is in the first column Routh's table and Limit[] returns DirectedInfinity[] *)
   If[
      Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
      fcol = (fcol /. DirectedInfinity[arg_] -> arg);
   ];
   
   If[
      Check[
         (* plot stable area *)
         graphics = InequalityPlot3D[
            And @@ ((# >= 0) & /@ fcol), {vars[[1]]}, {vars[[2]]},{vars[[3]]},
            DisplayFunction -> Identity, PlotRange->All,
            FilterOptions[InequalityPlot3D,opts],
            AxesLabel->{ToString[vars[[1]]],ToString[vars[[2]]],ToString[vars[[3]]]}
         ],
         unbounded, InequalityPlot3D::"unbounded"
      ] === unbounded,
      (* then *)
      Message[StabilityPlot::unboudedstabarea,optDefaultRange];
      StabilityPlot3D[charpoly,
         Evaluate[
            Sequence @@ MapThread[List[#1, Sequence @@ #2]&, {vars, optDefaultRange}],
            opts
         ]
      ],
      (* else *)
      Show[graphics, DisplayFunction->$DisplayFunction]
   ]

];

(******************************************************************************)
(*          TStabilityPlot                                                    *)
(******************************************************************************)
TStabilityPlot[charpoly:Poly[pc_,var_,deg_], {p1_,lp1_,rp1_} ,{p2_,lp2_,rp2_}, opts___?OptionQ] :=
Module[{ fcol,
         vars = Union[Cases[Variables[pc], _Symbol, Infinity]],
         optPlotArea = PlotArea /. {opts} /. Options[StabilityPlot],
         iplot
       },
   If[ Length[vars] > 2, Message[StabilityPlot::errvars];Return[Null]];
   
(** Continuous or Discrete time ***********************************************)
   Switch[var, 
   (** Continuous time ********************************************************)     
      Global`$ContinuousVariable,
         (* Routh's table *)
         fcol = First/@RouthTable[charpoly];
         
         (* If zero is in the first column Routh's table and Limit[] returns DirectedInfinity[] *)
         If[
            Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
            fcol = (fcol /. DirectedInfinity[arg_] -> arg);
         ];
         
         (* Inequalities *)
         ineqs = # >= 0& /@ fcol
      ,   
   (** Discrete time **********************************************************)
      Global`$DiscreteVariable,
         (* Jury's table *)
         fcol = First /@ Take[JuryTable[charpoly], {4, 2(deg + 1), 2}];
         
         (* Inequalities *)
         ineqs = Prepend[(# < 0)& /@ Rest[fcol], First[fcol] > 0]
      ,
   (** Continuous time ********************************************************)     
      _,
         fcol = First/@RouthTable[charpoly];
         If[
            Or @@ (MatchQ[#, _DirectedInfinity]& /@ fcol),
            fcol = (fcol /. DirectedInfinity[arg_] -> arg);
         ];
         ineqs = (# >= 0) & /@ fcol
   ]; (* End Switch *)
(******************************************************************************)

   MapIndexed[
      If[
         Check[
            iplot = InequalityPlot[#1, {p1, lp1, rp1}, {p2, lp2, rp2},
               DisplayFunction->Identity,
               FilterOptions[InequalityPlot, opts],
               PlotRange -> {{lp1, rp1}, {lp2, rp2}},
               Fills -> {RGBColor[1, 3/(2 + #2[[1]]), 0]}
            ],
            err, InequalityPlot::"region"
         ] === err,
         (* Then *)
         ContourPlot[If[#1, 0, 1]//Evaluate, {p1,lp1,rp1}, {p2,lp2,rp2},
            ContourSmoothing -> False, Contours -> 1,
            FilterOptions[ContourPlot,opts]//Evaluate,
            PlotPoints -> {200, 200},
            PlotRange->All,
            ColorFunction -> (If[# > 0, GrayLevel[1], Hue[.5]]&),
            Frame -> False, Axes -> True
         ],
         (* Else *)
         Show[iplot,DisplayFunction->$DisplayFunction]
      ]& 
      , ineqs
   ]
   
]

End[]
 
EndPackage[]