(*************************************)
(* Generate random matrix or polynom *)
(*************************************)

BeginPackage["Polynomial`Random`","Polynomial`Object`"]

(* U S A G E *)
PMRandom::usage = "PMRandom[deg_Integer, dim_Integer, var_Symbol:Global`$GlovalVar, opts___?OptionQ] - generate square random polynomial matrix.
                    \n\nPMRandom[deg_Integer, row_Integer, col_Integer, var_Symbol:Global`$GlovalVar, opts___?OptionQ] - generate random polynomial matrix. 
                    \n\nPMRandom[var_Symbol:Global`$GlovalVar, opts___?OptionQ] - generate random polynomial matrix. 
                    \n\nPMRandom[a:_Symbol, deg_Integer, row_Integer, col_Integer, var_Symbol:Global`$GlovalVar, opts___?OptionQ] - generate common random polynomial matrix with parametr a. 
                    \n\n\t Options: RandomData -> {TypeNumber, Range}, Subscript -> True.                     
                  ";


PRandom::usage = "PRandom[deg_Integer, var_Symbol:Global`$GlovalVar, opts___?OptionQ] - generate random polynom.
                 ";

(* O P T I O N S *)
Options[PMRandom] = {RandomData -> {Integer, {-10, 10}}, Subscript -> True, 
                        RandomRow -> 5,
                        RandomCol -> 5,
                        RandomDeg -> 5,
                        Precision -> 16 (*$MachinePrecision !!v5.0*)
                    };
                        
Options[PRandom] = {RandomData -> {Integer, {-10, 10}}, Subscript -> True, Type -> "std",
                        RandomDeg :> Random[Integer,{0,10}], Precision -> 16 (* $MachinePrecision *)};

(* M E S S A G E *)
General::SymPRandom = " Input degree of the polynom must be EvenQ. ";
General::PRanBadType = " Bad option Type `1` of PRandom. ";

Begin["`Private`"]
(* Developer context *)
<<Developer`
(****************************)
(* Random Polynomial Matrix *)
(****************************)

PMRandom[deg:_Integer, row:_Integer, col:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
Module[
    {   opt1 = Sequence @@ (RandomData /. {opts} /. Options[PMRandom]),
        opt2 = Sequence @@ (Precision /. {opts} /. Options[PMRandom])
    },
   If[opt1 === Real || opt1 === Integer, opt1 = Sequence[opt1,1]];
   If[opt1 === Complex, opt1 = Sequence[opt1,1+I]]; 
   If[opt1 === Integer || MatchQ[{opt1}, {Integer, {_, _}}], opt2 = Sequence[]];

   PolyMat[
      Table[
         Table[
            Random[opt1,opt2], {row},{col}                
         ],
         {deg+1}
      ]//ToPackedArray
      , var, deg
   ]
];

PMRandom[deg:_Integer, dim:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
PMRandom[deg, dim, dim, var, opts];

PMRandom[opts___?OptionQ] :=
Module[
    {   opt1 = Sequence @@ (RandomData /. {opts} /. Options[PMRandom]),
        opt2 = Sequence @@ (Precision /. {opts} /. Options[PMRandom]),
        var = Global`$Variable,
        col, row, deg
    },

   col = Random[Integer, {1,Sequence @@ (RandomCol /. {opts} /. Options[PMRandom])}];
   row = Random[Integer, {1,Sequence @@ (RandomRow /. {opts} /. Options[PMRandom])}];
   deg = Random[Integer, {0,Sequence @@ (RandomDeg /. {opts} /. Options[PMRandom])}];   
   
   If[opt1 === Integer || MatchQ[{opt1}, {Integer, {_, _}}], opt2 = Sequence[]];

   PolyMat[
      Table[
         Table[
            Random[opt1,opt2], {row},{col}                
         ],
         {deg+1}
      ]//ToPackedArray
      , var, deg
   ]
];
   
PMRandom[{arow_Integer,acol_Integer},var:_Symbol:Global`$Variable, opts___?OptionQ] :=
Module[
    {   opt1 = Sequence @@ (RandomData /. {opts} /. Options[PMRandom]),
        opt2 = Sequence @@ (Precision /. {opts} /. Options[PMRandom]),
        var = Global`$Variable,        
        col, row, deg
    },

   col = acol;
   row = arow;
   deg = Random[Integer, {0,Sequence @@ (RandomDeg /. {opts} /. Options[PMRandom])}];   

   If[opt1 === Integer || MatchQ[{opt1}, {Integer, {_, _}}], opt2 = Sequence[]];

   
   PolyMat[
      Table[
         Table[
            Random[opt1,opt2], {row},{col}                
         ],
         {deg+1}
      ]//ToPackedArray
      , var, deg
   ]
];
   
PMRandom[{dim_Integer},var:_Symbol:$Variable, opts___?OptionQ] :=
PMRandom[{dim,dim}, var, opts];
   
PMRandom[a:_Symbol, deg:_Integer, row:_Integer, col:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
   If[(Subscript /. {opts} /. Options[PMRandom]), 
   (* then *)
      PolyMat[
         Table[
            Table[
               Subscript[a, {n, {i, j}}], {i,row},{j,col}
            ],
            {n,0,deg}
         ]
         , var, deg
      ]
    ,
   (* else *)
      PolyMat[
         Table[
            Table[
               a[n, {i, j}], {i,row},{j,col}                
            ],
            {n,0,deg}
         ]
         , var, deg
      ]      
   ];


PMRandom[a:_Symbol, deg:_Integer, dim:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
PMRandom[a, deg, dim, dim, var, opts];

(*******************)
(* Random Polynoms *)
(*******************)
PRandom[deg:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
With[{typeopt = Type /. {opts} /. Options[PRandom]},
   Switch[typeopt,
      "std",Return[StdPRandom[deg,var,opts]],
      "sym",Return[SymPRandom[deg,var,opts]],
      _,Message[General::PRanBadType,typeopt]
   ];
];

PRandom[sym:_Symbol, deg:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
With[{typeopt = Type /. {opts} /. Options[PRandom]},
   Switch[typeopt,
      "std",Return[StdPRandom[sym,deg,var,opts]],
      "sym",Return[SymPRandom[sym,deg,var,opts]],
      _,Message[General::PRanBadType,typeopt]
   ]
];

PRandom[var:_Symbol:Global`$Variable, opts___?OptionQ] :=
With[{typeopt = Type /. {opts} /. Options[PRandom]},
   Switch[typeopt,
      "std",Return[StdPRandom[RandomDeg /. {opts} /. Options[PRandom],var,opts]],
      "sym",Return[SymPRandom[2*RandomDeg /. {opts} /. Options[PRandom],var,opts]],
      _,Message[General::PRanBadType,typeopt]
   ]
];

(**************************************)
(********* Standard polynoms **********)
StdPRandom[deg:_Integer, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
Module[{opt1 = Sequence @@ (RandomData /. {opts} /. Options[PRandom]),
        opt2 = Sequence @@ (Precision /. {opts} /. Options[PRandom]),
        list, last},
   If[opt1 === Real || opt1 === Integer, opt1 = Sequence[opt1,1]];
   If[opt1 === Complex, opt1 = Sequence[opt1,1+I]]; 
        
   If[opt1 === Integer || MatchQ[{opt1}, {Integer, {_, _}}], opt2 = Sequence[]];
           
   list = ToPackedArray[Table[ Random[opt1, opt2], {i, 0, deg}]];
   (* if last element in the list is zero; generate nonzero value *)
   If[ Last[list] === 0,
       While[
           (last = Random[opt1,opt2]) === 0
       ];
       list[[deg+1]] = last;
   ];
   Poly[list, var, deg]
];

StdPRandom[sym:_Symbol, deg:_Integer, var:_Symbol:Global`$Variable, opt___?OptionQ] :=
   Poly[
      If[(Subscript /. {opt} /. Options[PRandom]),
         Table[ Subscript[sym, i],{i, 0, deg}],
         Table[ Sym[sym, i],{i, 0, deg}]
      ], var, deg
   ];
   
(********* Symmetric polynoms **********)
SymPRandomTest[x : _Integer] := If[EvenQ[x], True, Message[General::SymPRandom]; False];

SymPRandom[sym:_Symbol, deg:_?SymPRandomTest, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
   Poly[
      If[(Subscript /. {opts} /. Options[PRandom]), 
         Table[ If[OddQ[i], 0, Subscript[sym, i]], {i, 0, deg}],
         Table[ If[OddQ[i], 0, Sym[sym, i]], {i, 0, deg}]
      ], var, deg
   ];

SymPRandom[deg_?SymPRandomTest, var:_Symbol:Global`$Variable, opts___?OptionQ] :=
Module[{opt1 = Sequence @@ (RandomData /. {opts} /. Options[PRandom]),
        opt2 = Sequence @@ (Precision /. {opts} /. Options[PRandom]),
        list, last},
   If[opt1 === Real || opt1 === Integer, opt1 = Sequence[opt1,1]];
   If[opt1 === Complex, opt1 = Sequence[opt1,1+I]]; 
        
   If[opt1 === Integer || MatchQ[{opt1}, {Integer, {_, _}}], opt2 = Sequence[]];        
        
   list = ToPackedArray[Table[ If[OddQ[i], 0, Random[opt1,opt2]], {i, 0, deg}]];
   If[ Last[list] === 0,
       While[
           (last = Random[opt1,opt2]) === 0
       ];
       list[[deg/2]] = last;
   ];
   Poly[list, var, deg]
];

(**********************************)
(* U S E F U L    F U N C T I O N *)
(**********************************)
Sym[a_Symbol, i_Integer] := ToExpression[ToString[a] <> ToString[i]];


End[ ]

EndPackage[]