(*********************************)
(*  S T A T E   P O R T R A I T  *)
(*********************************)
(* State portrait of a nonlinear systems. *)
(* G L O B A L      V A R I A B L E *)
$vsfnc = x; (* Variable of State Function *)

BeginPackage["Polynomial`StatePortraitNL`","Graphics`PlotField`","Utilities`FilterOptions`"]
(* U S A G E *)
StateDescription::usage = "StateDescription[]";
StatePortraitNL::usage = "StatePortrait[]";
StatePortraitPlot::usage = "StatePortraitPlot[]";
StatePortraitVector::usage = "StatePortraitVector[]";

(* M E S S A G E *)
General::badopt = " Bad options. ";

(* O P T I O N S *)
Options[StateDescription] = {Subscript -> False};
Options[StatePortraitNL] = {Subscript -> False};


Begin["`Private`"]

num = (_Integer | _Real);
nodisp = DisplayFunction -> Identity;
disp = DisplayFunction -> $DisplayFunction;

(***************)

StatePortraitNL[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},opts___?OptionQ] := 
StatePortraitNL[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{-10,10,1},{-10,10,1}},opts];

StatePortraitNL[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},
{{rx1:num,rx2:num,rxdi:num:1},{ry1:num,ry2:num,rydi:num:1}},opts___?OptionQ] :=
With[{
     optSPPlot = FilterOptions[ParametricPlot, opts],
     optSPVector = FilterOptions[PlotVectorField, opts]
     },
      Show[
        {
            StatePortraitPlot[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{rx1,rx2,rxdi},{ry1,ry2,rydi}},optSPPlot, nodisp],
            StatePortraitVector[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{rx1,rx2,rxdi},{ry1,ry2,rydi}},optSPVector, nodisp]
        },disp
      ]

];

StatePortraitNL[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},iniconditions:{{num, num} ..},opts___?OptionQ] := 
StatePortraitNL[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{-10,10,1},{-10,10,1}},iniconditions,opts];

StatePortraitNL[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},
{{rx1:num,rx2:num,rxdi:num:1},{ry1:num,ry2:num,rydi:num:1}}, iniconditions:{{num, num} ..},opts___?OptionQ] :=
With[{
     optSPPlot = FilterOptions[ParametricPlot, opts],
     optSPVector = FilterOptions[PlotVectorField, opts]
     },
      Show[
        {
            StatePortraitPlot[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{rx1,rx2,rxdi},{ry1,ry2,rydi}},optSPPlot, nodisp],
            StatePortraitVector[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{rx1,rx2,rxdi},{ry1,ry2,rydi}},optSPVector, nodisp],
            StatePortraitPlot[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax}, iniconditions, nodisp]
        },disp
      ]

];

(***************)

StatePortraitPlot[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},opts___?OptionQ] := 
StatePortraitPlot[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{-10,10,1},{-10,10,1}},opts];

StatePortraitPlot[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},
{{rx1:num,rx2:num,rxdi:num:1},{ry1:num,ry2:num,rydi:num:1}},opts___?OptionQ] :=
Module[
    {eq1 = aeq1, eq2 = aeq2, fnc1 = afnc1, fnc2 = afnc2, var = avar, data},
    
    Off[NDSolve::"ndsz"]; Off[InterpolatingFunction::"dmval"];
    
        data = 
            Level[
                Table[
                    NDSolve[
                        {eq1, eq2, fnc1[0] == i, fnc2[0] == j},
                        {fnc1[var], fnc2[var]}, {var, 0, tmax}
                    ][[1]]
                    , {i, rx1, rx2, rxdi}, {j, ry1, ry2, rydi}
                ],
            {2}];   

        Return[
            ParametricPlot[        (* ! ! Evaluate[{ ..}, {var, 0, tmax}] *)
                Evaluate[{fnc1[var], fnc2[var]} /. data,
                {var, 0, tmax}], opts, PlotRange -> {{rx1-2*rxdi, rx2+2*rxdi}, {ry1-2*rydi, ry2+2*rydi}}
            ]
        ]
];

(***************)

StatePortraitVector[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},opts___?OptionQ] := 
StatePortraitVector[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax},{{-10,10,1},{-10,10,1}},opts];

StatePortraitVector[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},
{{rx1:num,rx2:num,rxdi:num:1},{ry1:num,ry2:num,rydi:num:1}},opts___?OptionQ] :=
Module[
    {eq1 = aeq1, eq2 = aeq2, fnc1 = afnc1, fnc2 = afnc2, var = avar},

    Return[
        PlotVectorField[#[[2]] &  /@ {eq1,eq2}, {fnc1[var], rx1, rx2, rxdi}, {fnc2[var], ry1, ry2, rydi},
                opts, PlotRange -> {{rx1-2*rxdi, rx2+2*rxdi}, {ry1-2*rydi, ry2+2*rydi}}, ColorFunction -> Hue
            ]
    ]

];

(***************)

StatePortraitPlot[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num}, {x0:num,y0:num}, opts___?OptionQ] := 
StatePortraitPlot[{aeq1,aeq2}, {afnc1,afnc2}, {avar, tmax}, {{x0,y0}}, opts];

StatePortraitPlot[{aeq1_Equal,aeq2_Equal}, {afnc1_Symbol,afnc2_Symbol}, {avar_Symbol, tmax:num},
iniconditions:{{num, num} ..}, opts___?OptionQ] :=
Module[
    {eq1 = aeq1, eq2 = aeq2, fnc1 = afnc1, fnc2 = afnc2, var = avar, data},

    Off[NDSolve::"ndsz"]; Off[InterpolatingFunction::"dmval"];
    
        data = 
            Table[
                NDSolve[
                    {eq1, eq2, fnc1[0] == iniconditions[[i,1]], fnc2[0] == iniconditions[[i,2]]},
                    {fnc1[var], fnc2[var]}, {var, 0, tmax}
                ][[1]]
                ,{i, 1, Length[iniconditions]}
            ];
            
        Return[
            ParametricPlot[        (* ! ! Evaluate[{ ..}, {var, 0, tmax}] *)
                Evaluate[{fnc1[var], fnc2[var]} /. data,
                {var, 0, tmax}], opts, PlotStyle -> {{Thickness[.005], Hue[1]}}
            ]
        ]
];

(***************)

StateDescription[aeq_Equal, afnc_Symbol, avar_Symbol, asfnc:_Symbol:Global`$vsfnc, aopt___?OptionQ] :=
Module[
    {eq = aeq, fnc = afnc, sfnc = asfnc, var = avar, n, sym, opt = aopt}, 
    
    (* tests option argument *)
    If[
        Not[Element[Subscript /. {opt} /. Options[StateDescription],Booleans]],
        Message[General::badopt]; opt = Options[StateDescription][[1]]
    ];
    (*  *)
    If[
        Subscript /. {opt} /. Options[StateDescription],       
        sym[a_, i_] := Subscript[a, i],
        sym[a_, i_] := ToExpression[ToString[a] <> ToString[i]]
    ];

    n = Cases[eq, Derivative[r_][fnc][var] -> r, {0, Infinity}] // Max;
    Join[
        Table[D[sym[sfnc, i][var], var] == sym[sfnc, i + 1][var], {i, 1, n - 1}],
        {Equal @@ 
            Solve[ eq /. Table[
                                D[fnc[var], {var, i}] -> sym[sfnc, i + 1][var], {i, 0, n - 1}
                         ] /. D[fnc[var], {var, n}] -> D[sym[sfnc, n][var], var],
                        D[sym[sfnc, n][var], var]
            ][[1, 1]]
        }
    ]   
];


End[ ]

EndPackage[] 